open Syntax
open Printf
open List

let rec show_obj = function
  | IntObj i -> Printf.sprintf "%d" i
  | FunObj _ -> "fun"
  | NormObj _ -> "obj"
  | NilObj -> "nil"
  | TrueObj -> "true"
  | FalseObj -> "false"
  | NativeFunObj (arity,va_arg, _) -> sprintf "(native %d %b)" arity va_arg
  | Undef -> "undef"
  | ContObj _ -> "continuation"
  | CallCCObj -> "callcc"
  | ListObj l -> "[" ^ (String.concat ", " (map show_obj l)) ^ "]"
  | BoundFunObj _ -> "bound"
  | StringObj str -> sprintf "\"%s\"" str
      
let rec show_op = function
  | O_VRef ident -> sprintf "VRef(%s)"  ident 
  | O_VDef ident -> sprintf "VDef(%s)" ident
  | O_Call(i,tail) -> sprintf "Call(%d,%b)" i tail
  | O_IntLit i -> sprintf "Int(%d)" i 
  | O_SGet ident -> sprintf "SGet(%s)" ident
  | O_SSet ident -> sprintf "SSet(%s)" ident
  | O_Assign ident -> sprintf "%s" ident 
  | O_MkFun (_,_,ops) -> sprintf "MkFun[%s]" (show_ops ops)
  | O_Pop -> "Pop"
  | O_Stop -> "Stop"
  | O_Nop -> "Nop" 
  | O_Push obj -> sprintf "Push(%s)" (show_obj obj)
  | O_ExCall(i,tail) -> sprintf "ExCall(%d,%b)" i tail

and show_ops ops =
  String.concat " " (map show_op ops)

let rec show_env = function
  | ToplevelContext -> ""
  | Context(outer,tbl) ->
      (Hashtbl.fold (fun k v s -> sprintf "%s=%s,%s" k (show_obj v) s) tbl "") ^
        "|" ^ (show_env outer)

let print_objs objs =
  print_endline (">" ^ (String.concat ", " (map show_obj objs)))
