(* header *)
{
open Parser

let keyword = function
  | "function" -> FUNCTION
  | "var" -> VARDEF
  | "return" -> RETURN
  | "if" -> IF
  | "else" -> ELSE
  | "elsif" -> ELSIF
  | "do" -> DO
  | "def" -> DEF
  | "class" -> CLASS
  | s -> IDENT s
      
let last_token = ref FUNCTION
let return_token tok = begin
  last_token := tok;
  tok
end

let string_lit str =
  let cut = String.sub str 1 ((String.length str)-2) in
    cut
    
let newline parse lexbuf =
  match !last_token with
      PLUS | MINUS | EQUAL | EQEQ | LPAREN | COMMA | DOT -> parse lexbuf
    | _ -> STMT_TERM
}

(* rule *)
let identstart =
  ['A'-'Z' 'a'-'z' '_']
let identbody =
  ['A'-'Z' 'a'-'z' '_' '0'-'9']
let blank =
  [' ' '\t' '\r']
let dec_literal =
  [ '0' - '9' ] +
let string =
  '"' [ ^ '"']* '"'
rule token = parse
  |  blank +
      { token lexbuf }
  | string { return_token (STRING (string_lit (Lexing.lexeme lexbuf))) }
  | '\n'
      { newline token lexbuf }
  | '#' [ ^ '\n' ] * '\n'
      { newline token lexbuf }
  | '#' [ ^ '\n' ] * eof
      { return_token EOF }
  | identstart identbody *
      { return_token (keyword (Lexing.lexeme lexbuf)) }
  | dec_literal
      { return_token (INT_LIT (int_of_string (Lexing.lexeme lexbuf)))}
  | '+' { return_token PLUS }
  | '-' { return_token MINUS }
  | "==" { return_token EQEQ }
  | "=" { return_token EQUAL }
  | '(' { return_token LPAREN }
  | ')' { return_token RPAREN }
  | '{' { return_token LBRACE }
  | '}' { return_token RBRACE }
  | '[' { return_token LBRACKET }
  | ']' { return_token RBRACKET }
  | "," { return_token COMMA }
  | ";" { return_token STMT_TERM }
  | '.' { return_token DOT }
  | "<=" { return_token LESSEQUAL }
  | ">=" { return_token GREATEREQUAL }
  | '<' { return_token LESS }
  | '>' { return_token GREATER }
  | '*' { return_token STAR }
  | '/' { return_token SLASH }
  | "!" { return_token EXCLAMATION }
  | '@' { return_token AT }
  | "&&" { return_token ANPERANPER }
  | "||" { return_token BARBAR }
  | eof { return_token EOF }

    
