
%{
  open Syntax
%}

%token <string> IDENT
%token <int> INT_LIT
%token <string> STRING
  
%token EOF

/* keywords */
%token FUNCTION
%token VARDEF
%token RETURN
%token IF
%token ELSE
%token ELSIF
%token DO
%token DEF
%token CLASS
  
/* operators */
%token PLUS
%token MINUS
%token EQUAL
%token EQEQ
%token LESS
%token GREATEREQUAL
%token LESSEQUAL
%token GREATER
%token STAR
%token AT
%token SLASH
%token BARBAR
%token ANPERANPER
%token EXCLAMATION
  
/* structual symbols */
%token LPAREN
%token RPAREN
%token LBRACE
%token RBRACE
%token LBRACKET
%token RBRACKET
%token COMMA
%token STMT_TERM
%token DOT

/* precedences */
%right EQUAL
%right BARBAR
%right ANPERANPER
%nonassoc EQEQ LESS GREATER LESSEQUAL GREATEREQUAL
%left PLUS MINUS
%left STAR SLASH

%right EXCLAMATION
%nonassoc prec_unary_minus
%right LPAREN /* funcall */
%left DOT

  
%start main
%type <Syntax.exp list> main
%%

main:
  stmts EOF { $1 }
  
stmts:
  stmt STMT_TERM stmts { $1::$3 }
| STMT_TERM stmts { $2 }
| stmt { [$1] }
| /* empty */ { [] }
;

stmt:
  exp { $1 }
| vardef { $1 }
| FUNCTION IDENT LPAREN params RPAREN LBRACE stmts RBRACE {
    Vardef($2, MakeFun(fst $4, snd $4, $7))
  }
| DEF IDENT LPAREN params RPAREN LBRACE stmts RBRACE {
    SSet(SGet(Varref("self"),"_methods_"),$2,
         MakeFun("self"::(fst $4), snd $4, $7))
  }
| CLASS IDENT super LBRACE stmts RBRACE {
    Class($2, $3, $5)
  }
| RETURN exp { Return($2) }
;
super:
  LESS exp { Some $2 }
| /* empty */ { None }
      
exp:
  varref { $1 }
| int_lit { $1 }
| call { $1 }
| call_with_fun { $1 }
| sget { $1 }
| sset { $1 }
| self_sget { $1 }
| self_sset { $1 }
| assign { $1 }
| mkfun { $1 }
| LPAREN exp RPAREN { $2 }
| list { $1 }
| if_stmt { $1 }
| exp PLUS exp { Call(SGet($1, "+"), [$3], None) }
| exp MINUS exp { Call(SGet($1, "-"), [$3], None) }
| exp STAR exp { Call(SGet($1, "*"), [$3], None) }
| exp SLASH exp { Call(SGet($1, "/"), [$3], None) }
| exp EQEQ exp { Call(SGet($1, "=="), [$3], None) }
| exp LESS exp { Call(SGet($1, "<"), [$3], None) }
| exp GREATER exp { Call(SGet($1, ">"), [$3], None) }
| exp LESSEQUAL exp { Call(SGet($1, "<="), [$3], None) }
| exp GREATEREQUAL exp { Call(SGet($1, ">="), [$3], None) }
| exp ANPERANPER exp { And($1,$3) }
| exp BARBAR exp { Or($1,$3) }
| EXCLAMATION exp { Not($2) }
| MINUS exp %prec prec_unary_minus { Call(SGet($2, "-@"), [], None) }
| STRING { String($1) }
;

if_stmt:
  IF LPAREN exp RPAREN LBRACE stmts RBRACE if_tail { IfStmt($3, $6, $8) }
;
if_tail:
  ELSE LBRACE stmts RBRACE { $3 }
| ELSIF LPAREN exp RPAREN LBRACE stmts RBRACE if_tail { [IfStmt($3, $6, $8)] }
| { [] }
;
varref:
  IDENT { Varref $1 }
;

call:
   exp LPAREN args RPAREN  { Call($1, fst $3, snd $3) }
;

call_with_fun:
  exp LPAREN args RPAREN DO LPAREN params RPAREN LBRACE stmts RBRACE
  { Call($1, (fst $3) @ [MakeFun(fst $7,snd $7, $10)], snd $3) }
;
args:
  norm_args { ($1,None) }
| norm_args COMMA STAR exp { ($1, Some $4) }
| STAR exp { ([], Some $2) }
| { ([],None) }
;
norm_args:
  norm_args COMMA exp { $1 @ [$3] }
| exp { [$1] }
;

int_lit:
  INT_LIT { IntLit $1 }
;

sget:
  exp DOT IDENT { SGet ($1, $3) }
;

sset:
  exp DOT IDENT EQUAL exp { SSet($1, $3, $5)}
;

self_sget:
  AT IDENT { SGet( Varref("self"), $2) }
;
self_sset:
  AT IDENT EQUAL exp { SSet( Varref("self"), $2, $4) }
;

assign:
  IDENT EQUAL exp { Assign ($1, $3) }
;

vardef:
  VARDEF IDENT EQUAL exp { Vardef ($2, $4) }
;

mkfun:
  FUNCTION LPAREN params RPAREN LBRACE stmts RBRACE {
    MakeFun (fst $3, snd $3, $6)
  }
;

params:
  nparams { ($1,None) }
| nparams COMMA STAR IDENT { ($1, Some $4) }
| STAR IDENT { ([], Some $2) }
| /* empty */ { ([], None) }
;
nparams:
  nparams COMMA IDENT { $1 @ [$3] }
| IDENT { [$1] }
;

list:
  LBRACKET list_elems RBRACKET { MakeList $2 }
;
list_elems:
  exp COMMA list_elems { $1::$3 }
| exp { [$1] }
| { [] }
;
%%

  
