open Syntax
open Eval
open Compile
open Debug

let rec show_tree stmts =
  let rec show_exp = function
    | Varref v -> Printf.sprintf "(var %s)" v
    | Call (exp,args,_) ->
        let arg = String.concat "" (List.map show_exp args) in
          Printf.sprintf "(call [%s][%s])" (show_exp exp) arg
    | IntLit i -> Printf.sprintf "(int %d)" i
    | SGet (exp,ident) -> 
        Printf.sprintf "(sget %s[%s])" (show_exp exp) ident
    | Assign (ident, exp) ->
        Printf.sprintf "(assgin %s <- %s)" ident (show_exp exp)
    | MakeFun (params, rparam, stmts) ->
        let params_str = String.concat "," params in
          Printf.sprintf "(function [%s]{\n%s\n})" params_str (show_tree stmts)
    | Vardef (ident,exp) ->
        Printf.sprintf "(vardef %s <- %s)" ident (show_exp exp)
  in
    String.concat "\n" (List.map show_exp stmts)

let make_tree channnel =
  let lexbuf = Lexing.from_channel channnel in
    Parser.main Lexer.token lexbuf
      
let print_tree () =
  print_endline (show_tree (make_tree stdin))


(* let _ = print_tree () *)
let _ =
  let context = new_context () in
  let ops = (compile_exps (make_tree stdin)) in
    begin
      (* print_endline (show_ops ops); *)
      eval_ops (ops @ [O_Stop]) context []
    end
